#include <stdint.h>
#include <memory.h>
#include <vga.h>
#include <debug.h>
#include <multiboot.h>
#include <interrupts.h>

struct kernel_boot_data_st kernel_boot_data;

int divide(int a, int b)
{
    return a/b;
}

void kmain(uint64_t multiboot_magic, void *multiboot_data)
{
    vga_init();

    debug("Started kernel\n");

    debug("Multiboot magic: %x", multiboot_magic);
    multiboot_init(multiboot_magic, P2V(multiboot_data));

    debug_info("Multiboot v %d kernel loaded.\n", kernel_boot_data.multiboot_version);
    debug_info("Bootloader: %s\n", kernel_boot_data.bootloader);

    debug_info("Memory areas:\n");
    size_t index = 0;
    uintptr_t start, end;
    uint32_t type;
    while(!multiboot_get_memory_area(index++, &start, &end, &type))
    {
        debug_info("%d %x-%x\n", type, start, end);
    }

    interrupt_init();

    debug("Hello, world!");

    // Force a divide by zero error
    divide(5, 0);

    PANIC("End of kernel function!");
    
    debug_info("Broke out of panic");
    for(;;);
}